@@ -156,9 +156,10 @@ pub trait Deqp {
                         /* Remove the reported test from our list of tests to run.  If it's not in our list, then it's
                          * a subtest.
                          */
-                        if let Some(position) = remaining_tests.iter().position(|x| {
-                            x.name() == result.test.strip_prefix(self.prefix()).unwrap()
-                        }) {
+                        if let Some(position) = remaining_tests
+                            .iter()
+                            .position(|x| x.name() == result.test.trim_start_matches(self.prefix()))
+                        {
                             remaining_tests.swap_remove(position);
                         } else if !result.subtest {
                             error!(
