@@ -1,5 +1,6 @@
 use crate::parse_deqp::DeqpStatus;
 use anyhow::{Context, Result};
+use log::*;
 use regex::Regex;
 use std::collections::HashMap;
 use std::fmt;
@@ -9,6 +10,31 @@ use std::str::FromStr;
 use std::time::{Duration, Instant};
 use structopt::StructOpt;
 
+// Wrapper for displaying a duration in h:m:s (integer seconds, rounded down)
+struct HMSDuration(Duration);
+impl fmt::Display for HMSDuration {
+    fn fmt(&self, f: &mut fmt::Formatter) -> fmt::Result {
+        let mut secs = self.0.as_secs();
+        let mut has_hours = false;
+        if secs >= 3600 {
+            write!(f, "{}:", secs / 3600)?;
+            secs %= 3600;
+            has_hours = true;
+        }
+        if secs >= 60 {
+            if has_hours {
+                write!(f, "{:02}:", secs / 60)?;
+            } else {
+                write!(f, "{}:", secs / 60)?;
+            }
+            secs %= 60;
+            write!(f, "{:02}", secs)
+        } else {
+            write!(f, "{}", secs)
+        }
+    }
+}
+
 #[derive(Debug, Clone, Copy, PartialEq)]
 pub enum RunnerStatus {
     Pass,
@@ -384,6 +410,38 @@ impl RunnerResults {
         Ok(results)
     }
 
+    pub fn status_update<W: Write>(&self, writer: &mut W, total_tests: u32) {
+        let duration = self.time.elapsed();
+
+        write!(
+            writer,
+            "{}, Duration: {duration}",
+            self.result_counts,
+            duration = HMSDuration(duration),
+        )
+        .context("status update")
+        .unwrap_or_else(|e| error!("{}", e));
+
+        // If we have some tests completed, use that to estimate remaining runtime.
+        let duration = duration.as_secs_f32();
+        if self.result_counts.total != 0 {
+            let average_test_time = duration / self.result_counts.total as f32;
+            let remaining = average_test_time * (total_tests - self.result_counts.total) as f32;
+
+            write!(
+                writer,
+                ", Remaining: {}",
+                HMSDuration(Duration::from_secs_f32(remaining))
+            )
+            .context("status update")
+            .unwrap_or_else(|e| error!("{}", e));
+        }
+
+        writeln!(writer)
+            .context("status update")
+            .unwrap_or_else(|e| error!("{}", e));
+    }
+
     pub fn print_summary(&self, summary_limit: usize) {
         if self.tests.is_empty() {
             return;
@@ -586,6 +644,18 @@ dEQP-GLES2.info.version,Pass"
     fn csv_test_missing_status() {
         assert!(RunnerResults::from_csv(&mut "dEQP-GLES2.info.version".as_bytes()).is_err());
     }
+
+    #[test]
+    fn hms_display() {
+        assert_eq!(format!("{}", HMSDuration(Duration::new(15, 20))), "15");
+        assert_eq!(format!("{}", HMSDuration(Duration::new(0, 20))), "0");
+        assert_eq!(format!("{}", HMSDuration(Duration::new(70, 20))), "1:10");
+        assert_eq!(format!("{}", HMSDuration(Duration::new(69, 20))), "1:09");
+        assert_eq!(
+            format!("{}", HMSDuration(Duration::new(3735, 20))),
+            "1:02:15"
+        );
+    }
 }
 
 #[derive(Debug, StructOpt)]
